import * as ec2 from '@aws-cdk/aws-ec2';
import * as kms from '@aws-cdk/aws-kms';
import * as lambda from '@aws-cdk/aws-lambda';
import * as sm from '@aws-cdk/aws-secretsmanager';
import * as cdk from '@aws-cdk/core';
/**
 * Properties for a newly created Grafana Handler Construct.
 *
 * A valid Grafana dashboard JSON has an uid, id and title key in the root of the object.
 * We generate these based on input so static JSON files are not a problem when wanting to deploy more dynamic
 * but the end result is still deterministic. This is all derived from the dashboardAppName property
 *
 * @stability stable
 */
export interface GrafanaHandlerProps {
    /**
     * The secret in SecretsManager containing your Grafana password.
     *
     * If needed, specify an optional grafanaPwSecretKey to fetch a value for a specific JSON key in the Secret value.
     *
     * @stability stable
     */
    readonly grafanaPwSecret: sm.ISecret;
    /**
     * The optional key to be looked up from your Grafana password secret in Secretsmanager.
     *
     * @stability stable
     */
    readonly grafanaPwSecretKey?: string;
    /**
     * The name of the S3 bucket containing your dashboard file.
     *
     * @stability stable
     */
    readonly bucketName: string;
    /**
     * The object key in where you stored your dashboard file under.
     *
     * @stability stable
     */
    readonly objectKey: string;
    /**
     * A unique identifier to identify this dashboard in Grafana.
     *
     * This identifier is used to set or overwrite the title, id and uid keys in the dashboard json file
     * The identifier should be unique!
     *
     * @stability stable
     */
    readonly dashboardAppName: string;
    /**
     * The path to your local dashboard file.
     *
     * Give it in so the Construct can calculate an MD5 hash of it. This is needed as otherwise CloudFormation would not know when to redeploy your dashboard to Grafana when it changes.
     *
     * @stability stable
     */
    readonly localFilePath: string;
    /**
     * @stability stable
     */
    readonly grafanaUrl: string;
    /**
     * @stability stable
     */
    readonly timeout?: cdk.Duration;
    /**
     * @stability stable
     */
    readonly vpcSubnets?: ec2.SubnetSelection;
    /**
     * @stability stable
     */
    readonly vpc?: ec2.IVpc;
    /**
     * @stability stable
     */
    readonly kmsKey?: kms.IKey;
}
/**
 * @stability stable
 */
export declare class GrafanaHandler extends cdk.Construct {
    /**
     * @stability stable
     */
    readonly grafanaHandlerFunction: lambda.SingletonFunction;
    /**
     * @stability stable
     */
    readonly grafanaFunctionCRHandler: cdk.CustomResource;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, id: string, props: GrafanaHandlerProps);
}
